from typing import Any

from ..client import A79Client
from ..models.tools import DEFAULT
from ..models.tools.composio_models import ComposioResult
from ..models.tools.gmail_models import (
    AddLabelToEmailInput,
    CreateEmailDraftInput,
    CreateLabelInput,
    DeleteDraftInput,
    DeleteMessageInput,
    FetchEmailsInput,
    FetchMessageByMessageIdInput,
    FetchMessageByThreadIdInput,
    GetAttachmentInput,
    GetContactsInput,
    GetPeopleInput,
    GetProfileInput,
    ListDraftsInput,
    ListLabelsInput,
    ListThreadsInput,
    ModifyThreadLabelsInput,
    MoveToTrashInput,
    PatchLabelInput,
    RemoveLabelInput,
    ReplyToThreadInput,
    SearchPeopleInput,
    SendDraftInput,
    SendEmailInput,
)

__all__ = [
    "AddLabelToEmailInput",
    "CreateEmailDraftInput",
    "CreateLabelInput",
    "DeleteDraftInput",
    "DeleteMessageInput",
    "FetchEmailsInput",
    "FetchMessageByMessageIdInput",
    "FetchMessageByThreadIdInput",
    "GetAttachmentInput",
    "GetContactsInput",
    "GetPeopleInput",
    "GetProfileInput",
    "ListDraftsInput",
    "ListLabelsInput",
    "ListThreadsInput",
    "ModifyThreadLabelsInput",
    "MoveToTrashInput",
    "PatchLabelInput",
    "RemoveLabelInput",
    "ReplyToThreadInput",
    "SearchPeopleInput",
    "SendDraftInput",
    "SendEmailInput",
    "get_people",
    "fetch_emails",
    "fetch_message_by_thread_id",
    "search_people",
    "send_email",
    "create_email_draft",
    "fetch_message_by_message_id",
    "move_to_trash",
    "create_label",
    "patch_label",
    "get_attachment",
    "delete_message",
    "remove_label",
    "get_profile",
    "delete_draft",
    "add_label_to_email",
    "get_contacts",
    "reply_to_thread",
    "list_labels",
    "send_draft",
    "list_drafts",
    "list_threads",
    "modify_thread_labels",
]


def get_people(
    *,
    other_contacts: bool | None = DEFAULT,
    page_size: int | None = DEFAULT,
    page_token: str | None = DEFAULT,
    person_fields: str | None = DEFAULT,
    resource_name: str | None = DEFAULT,
    sync_token: str | None = DEFAULT,
) -> ComposioResult:
    """Execute Gmail: Get People"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = GetPeopleInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="get_people", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def fetch_emails(
    *,
    include_payload: bool | None = DEFAULT,
    include_spam_trash: bool | None = DEFAULT,
    label_ids: list[Any] | None = DEFAULT,
    max_results: int | None = DEFAULT,
    page_token: str | None = DEFAULT,
    query: str | None = DEFAULT,
    user_id: str | None = DEFAULT,
) -> ComposioResult:
    """Execute Gmail: Fetch Emails"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = FetchEmailsInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="fetch_emails", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def fetch_message_by_thread_id(
    *, page_token: str | None = DEFAULT, thread_id: str, user_id: str | None = DEFAULT
) -> ComposioResult:
    """Execute Gmail: Fetch Message By Thread Id"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = FetchMessageByThreadIdInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="fetch_message_by_thread_id", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def search_people(
    *,
    other_contacts: bool | None = DEFAULT,
    pageSize: int | None = DEFAULT,
    person_fields: str | None = DEFAULT,
    query: str,
) -> ComposioResult:
    """Execute Gmail: Search People"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = SearchPeopleInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="search_people", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def send_email(
    *,
    attachment: dict[str, Any] | None = DEFAULT,
    bcc: list[Any] | None = DEFAULT,
    body: str,
    cc: list[Any] | None = DEFAULT,
    extra_recipients: list[Any] | None = DEFAULT,
    is_html: bool | None = DEFAULT,
    recipient_email: str,
    subject: str | None = DEFAULT,
    user_id: str | None = DEFAULT,
) -> ComposioResult:
    """Execute Gmail: Send Email"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = SendEmailInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="send_email", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def create_email_draft(
    *,
    attachment: dict[str, Any] | None = DEFAULT,
    bcc: list[Any] | None = DEFAULT,
    body: str,
    cc: list[Any] | None = DEFAULT,
    extra_recipients: list[Any] | None = DEFAULT,
    is_html: bool | None = DEFAULT,
    recipient_email: str,
    subject: str,
    thread_id: str | None = DEFAULT,
    user_id: str | None = DEFAULT,
) -> ComposioResult:
    """Execute Gmail: Create Email Draft"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = CreateEmailDraftInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="create_email_draft", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def fetch_message_by_message_id(
    *, format: str | None = DEFAULT, message_id: str, user_id: str | None = DEFAULT
) -> ComposioResult:
    """Execute Gmail: Fetch Message By Message Id"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = FetchMessageByMessageIdInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail",
        name="fetch_message_by_message_id",
        input=input_model.model_dump(),
    )
    return ComposioResult.model_validate(output_model)


def move_to_trash(*, message_id: str, user_id: str | None = DEFAULT) -> ComposioResult:
    """Execute Gmail: Move To Trash"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = MoveToTrashInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="move_to_trash", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def create_label(
    *,
    label_list_visibility: str | None = DEFAULT,
    label_name: str,
    message_list_visibility: str | None = DEFAULT,
    user_id: str | None = DEFAULT,
) -> ComposioResult:
    """Execute Gmail: Create Label"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = CreateLabelInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="create_label", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def patch_label(
    *,
    color: dict[str, Any] | None = DEFAULT,
    id: str,
    labelListVisibility: str | None = DEFAULT,
    messageListVisibility: str | None = DEFAULT,
    name: str | None = DEFAULT,
    userId: str,
) -> ComposioResult:
    """Execute Gmail: Patch Label"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = PatchLabelInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="patch_label", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def get_attachment(
    *, attachment_id: str, file_name: str, message_id: str, user_id: str | None = DEFAULT
) -> ComposioResult:
    """Execute Gmail: Get Attachment"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = GetAttachmentInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="get_attachment", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def delete_message(*, message_id: str, user_id: str | None = DEFAULT) -> ComposioResult:
    """Execute Gmail: Delete Message"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = DeleteMessageInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="delete_message", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def remove_label(*, label_id: str, user_id: str | None = DEFAULT) -> ComposioResult:
    """Execute Gmail: Remove Label"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = RemoveLabelInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="remove_label", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def get_profile(*, user_id: str | None = DEFAULT) -> ComposioResult:
    """Execute Gmail: Get Profile"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = GetProfileInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="get_profile", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def delete_draft(*, draft_id: str, user_id: str | None = DEFAULT) -> ComposioResult:
    """Execute Gmail: Delete Draft"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = DeleteDraftInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="delete_draft", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def add_label_to_email(
    *,
    add_label_ids: list[Any] | None = DEFAULT,
    message_id: str,
    remove_label_ids: list[Any] | None = DEFAULT,
    user_id: str | None = DEFAULT,
) -> ComposioResult:
    """Execute Gmail: Add Label To Email"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = AddLabelToEmailInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="add_label_to_email", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def get_contacts(
    *,
    page_token: str | None = DEFAULT,
    person_fields: str | None = DEFAULT,
    resource_name: str | None = DEFAULT,
) -> ComposioResult:
    """Execute Gmail: Get Contacts"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = GetContactsInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="get_contacts", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def reply_to_thread(
    *,
    bcc: list[Any] | None = DEFAULT,
    cc: list[Any] | None = DEFAULT,
    extra_recipients: list[Any] | None = DEFAULT,
    is_html: bool | None = DEFAULT,
    message_body: str,
    recipient_email: str,
    thread_id: str,
    user_id: str | None = DEFAULT,
) -> ComposioResult:
    """Execute Gmail: Reply To Thread"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = ReplyToThreadInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="reply_to_thread", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def list_labels(*, user_id: str | None = DEFAULT) -> ComposioResult:
    """Execute Gmail: List Labels"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = ListLabelsInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="list_labels", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def send_draft(*, draft_id: str, user_id: str | None = DEFAULT) -> ComposioResult:
    """Execute Gmail: Send Draft"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = SendDraftInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="send_draft", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def list_drafts(
    *,
    max_results: int | None = DEFAULT,
    page_token: str | None = DEFAULT,
    user_id: str | None = DEFAULT,
) -> ComposioResult:
    """Execute Gmail: List Drafts"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = ListDraftsInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="list_drafts", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def list_threads(
    *,
    max_results: int | None = DEFAULT,
    page_token: str | None = DEFAULT,
    query: str | None = DEFAULT,
    user_id: str | None = DEFAULT,
) -> ComposioResult:
    """Execute Gmail: List Threads"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = ListThreadsInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="list_threads", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)


def modify_thread_labels(
    *,
    add_label_ids: list[Any] | None = DEFAULT,
    remove_label_ids: list[Any] | None = DEFAULT,
    thread_id: str,
    user_id: str | None = DEFAULT,
) -> ComposioResult:
    """Execute Gmail: Modify Thread Labels"""
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = ModifyThreadLabelsInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="gmail", name="modify_thread_labels", input=input_model.model_dump()
    )
    return ComposioResult.model_validate(output_model)



"""
Models for dataframe tools using pandas-ai
"""

from typing import Any

from pydantic import BaseModel, Field

from . import ToolOutput


class DataFrameQueryInput(BaseModel):
    """Input for querying a pandas DataFrame using natural language"""

    dataframe: Any = Field(description="The pandas DataFrame to query")
    query: str = Field(description="Natural language query to execute on the dataframe")

    class Config:
        arbitrary_types_allowed = True


class DataFrameQueryOutput(ToolOutput):
    """Output from DataFrame query execution"""

    result: Any = Field(
        description=(
            "The result of the query execution - can be a string, number, "
            "dataframe as dict, or dataframe"
        )
    )
    result_type: str = Field(
        description="The type of result returned (string, number, dataframe, list)"
    )
    query_executed: str = Field(description="The original query that was executed")



from typing import Any

from ..client import A79Client
from ..models.tools import DEFAULT
from ..models.tools.db_models import (
    DatabaseCredentials,
    DatabaseHierarchy,
    DatabaseSelection,
    Enum,
    ExecuteSQLInput,
    ExecuteSQLOutput,
    ListDatabasesInput,
    ListDatabasesOutput,
    ListSchemasInput,
    ListSchemasOutput,
    ListTablesInput,
    ListTablesOutput,
    NLToSQLInput,
    NLToSQLOutput,
    SchemaSelection,
    SearchTablesInput,
    SearchTablesOutput,
    SourceSelectionInput,
    SourceSelectionOutput,
    SQADatabaseConfig,
    SqlDialect,
    SqlType,
)

__all__ = [
    "DatabaseCredentials",
    "DatabaseHierarchy",
    "DatabaseSelection",
    "Enum",
    "ExecuteSQLInput",
    "ExecuteSQLOutput",
    "ListDatabasesInput",
    "ListDatabasesOutput",
    "ListSchemasInput",
    "ListSchemasOutput",
    "ListTablesInput",
    "ListTablesOutput",
    "NLToSQLInput",
    "NLToSQLOutput",
    "SQADatabaseConfig",
    "SchemaSelection",
    "SearchTablesInput",
    "SearchTablesOutput",
    "SourceSelectionInput",
    "SourceSelectionOutput",
    "SqlDialect",
    "SqlType",
    "generate_sql",
    "execute_sql",
    "list_databases",
    "list_schemas",
    "list_tables",
    "search_tables",
]


def generate_sql(
    *,
    query: str,
    num_tables_to_filter_for_sql_generation: int = DEFAULT,
    sample_rows: dict[str, list[dict[str, Any]]] | None = DEFAULT,
    connector_name: str,
) -> NLToSQLOutput:
    """
    Convert natural language queries to SQL with automatic table selection.

    This tool combines table selection and SQL generation into a single step:
    1. Analyzes your query to select the most relevant tables
    2. Generates optimized SQL using the selected tables
    3. Converts to the appropriate SQL dialect for your database
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = NLToSQLInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="nl_to_sql", name="generate_sql", input=input_model.model_dump()
    )
    return NLToSQLOutput.model_validate(output_model)


def execute_sql(
    *,
    sql_query: str,
    database_name: str,
    connector_name: str,
    limit: int | None = DEFAULT,
) -> ExecuteSQLOutput:
    """
    Execute a SQL query and return results as a pandas DataFrame-compatible format.

    This tool executes raw SQL queries against your database and returns the results
    in a format that can be easily converted to a pandas DataFrame. It supports all
    standard SQL operations including SELECT, JOIN, GROUP BY, etc.

    The results are returned as a list of dictionaries where each dictionary represents
    a row, making it easy to work with the data in Python.
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = ExecuteSQLInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="nl_to_sql", name="execute_sql", input=input_model.model_dump()
    )
    return ExecuteSQLOutput.model_validate(output_model)


def list_databases(*, connector_name: str) -> ListDatabasesOutput:
    """
    List all available databases from the specified connector.

    This tool returns a list of all databases accessible through the connector,
    useful for database discovery and exploration.
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = ListDatabasesInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="nl_to_sql", name="list_databases", input=input_model.model_dump()
    )
    return ListDatabasesOutput.model_validate(output_model)


def list_schemas(*, database_name: str, connector_name: str) -> ListSchemasOutput:
    """
    List all schemas in the specified database.

    This tool returns a list of all schemas within a database, helpful for
    understanding the database structure and organization.
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = ListSchemasInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="nl_to_sql", name="list_schemas", input=input_model.model_dump()
    )
    return ListSchemasOutput.model_validate(output_model)


def list_tables(
    *, database_name: str, schema_name: str, connector_name: str
) -> ListTablesOutput:
    """
    List all tables in the specified schema of a database.

    This tool returns a list of all tables within a schema, useful for
    exploring the data structure and finding specific tables.
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = ListTablesInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="nl_to_sql", name="list_tables", input=input_model.model_dump()
    )
    return ListTablesOutput.model_validate(output_model)


def search_tables(
    *, query: str, connector_name: str, num_tables: int = DEFAULT
) -> SearchTablesOutput:
    """
    Search for relevant tables using natural language queries.

    This tool uses semantic search to find tables that match your query,
    returning the most relevant tables with scores and explanations.
    It's particularly useful when you don't know the exact table names
    but know what kind of data you're looking for.
    """
    kwargs = locals()
    kwargs = {k: v for k, v in kwargs.items() if v is not DEFAULT}
    input_model = SearchTablesInput.model_validate(kwargs)

    client = A79Client()
    output_model = client.execute_tool(
        package="nl_to_sql", name="search_tables", input=input_model.model_dump()
    )
    return SearchTablesOutput.model_validate(output_model)



# This is a generated file by scripts/codegen/composio.py, do not edit manually
# ruff: noqa: E501  # Ignore line length issues in generated files
from typing import Any, Optional

from pydantic import BaseModel, Field


class GetPeopleInput(BaseModel):
    """Input model for GMAIL_GET_PEOPLE"""

    other_contacts: Optional[bool] = Field(
        default=False,
        description="If true, retrieves 'Other Contacts' (people interacted with but not explicitly saved), ignoring `resource_name` and enabling pagination/sync. If false, retrieves information for the single person specified by `resource_name`.",
    )  # noqa: E501

    page_size: Optional[int] = Field(
        default=10,
        description="The number of 'Other Contacts' to return per page. Applicable only when `other_contacts` is true.",
    )  # noqa: E501

    page_token: Optional[str] = Field(
        default="",
        description="An opaque token from a previous response to retrieve the next page of 'Other Contacts' results. Applicable only when `other_contacts` is true and paginating.",
    )  # noqa: E501

    person_fields: Optional[str] = Field(
        default="emailAddresses,names,birthdays,genders",
        description="A comma-separated field mask to restrict which fields on the person (or persons) are returned. Consult the Google People API documentation for a comprehensive list of valid fields.",
    )  # noqa: E501

    resource_name: Optional[str] = Field(
        default="people/me",
        description="Resource name identifying the person for whom to retrieve information (like the authenticated user or a specific contact). Used only when `other_contacts` is false.",
    )  # noqa: E501

    sync_token: Optional[str] = Field(
        default="",
        description="A token from a previous 'Other Contacts' list call to retrieve only changes since the last sync; leave empty for an initial full sync. Applicable only when `other_contacts` is true.",
    )  # noqa: E501


class FetchEmailsInput(BaseModel):
    """Input model for GMAIL_FETCH_EMAILS"""

    include_payload: Optional[bool] = Field(
        default=True,
        description="Set to true to include full message payload (headers, body, attachments); false for metadata only.",
    )  # noqa: E501

    include_spam_trash: Optional[bool] = Field(
        default=False,
        description="Set to true to include messages from 'SPAM' and 'TRASH'.",
    )  # noqa: E501

    label_ids: Optional[list[Any]] = Field(
        default=None,
        description="Filter by label IDs; only messages with all specified labels are returned. Common IDs: 'INBOX', 'SPAM', 'TRASH', 'UNREAD', 'STARRED', 'IMPORTANT', 'CATEGORY_PERSONAL', 'CATEGORY_SOCIAL', 'CATEGORY_PROMOTIONS', 'CATEGORY_UPDATES', 'CATEGORY_FORUMS'. Use 'listLabels' action for custom IDs.",
    )  # noqa: E501

    max_results: Optional[int] = Field(
        default=1, description="Maximum number of messages to retrieve per page."
    )  # noqa: E501

    page_token: Optional[str] = Field(
        default=None,
        description="Token for retrieving a specific page, obtained from a previous response's `nextPageToken`. Omit for the first page.",
    )  # noqa: E501

    query: Optional[str] = Field(
        default=None,
        description="Gmail advanced search query (e.g., 'from:user subject:meeting'). Supports operators like 'from:', 'to:', 'subject:', 'label:', 'has:attachment', 'is:unread', 'after:YYYY/MM/DD', 'before:YYYY/MM/DD', AND/OR/NOT. Use quotes for exact phrases. Omit for no query filter.",
    )  # noqa: E501

    user_id: Optional[str] = Field(
        default="me",
        description="User's email address or 'me' for the authenticated user.",
    )  # noqa: E501


class FetchMessageByThreadIdInput(BaseModel):
    """Input model for GMAIL_FETCH_MESSAGE_BY_THREAD_ID"""

    page_token: Optional[str] = Field(
        default="",
        description="Opaque page token for fetching a specific page of messages if results are paginated.",
    )  # noqa: E501

    thread_id: str = Field(
        description="Unique ID of the thread, obtainable from actions like 'listThreads' or 'fetchEmails'."
    )  # noqa: E501

    user_id: Optional[str] = Field(
        default="me", description="The email address of the user."
    )  # noqa: E501


class SearchPeopleInput(BaseModel):
    """Input model for GMAIL_SEARCH_PEOPLE"""

    other_contacts: Optional[bool] = Field(
        default=False,
        description="Include 'Other Contacts' (interacted with but not explicitly saved) in search results; if false, searches only primary contacts.",
    )  # noqa: E501

    pageSize: Optional[int] = Field(
        default=10,
        description="Maximum results to return; values >30 are capped to 30 by the API.",
    )  # noqa: E501

    person_fields: Optional[str] = Field(
        default="emailAddresses,names,phoneNumbers",
        description="Comma-separated fields to return (e.g., 'names,emailAddresses'); see PersonFields enum. If 'other_contacts' is true, only 'emailAddresses', 'names', 'phoneNumbers' are allowed.",
    )  # noqa: E501

    query: str = Field(
        description="Matches contact names, nicknames, email addresses, phone numbers, and organization fields."
    )  # noqa: E501


class SendEmailInput(BaseModel):
    """Input model for GMAIL_SEND_EMAIL"""

    attachment: Optional[dict[str, Any]] = Field(
        default=None,
        description="File to attach; ensure `s3key`, `mimetype`, and `name` are set if provided. Omit or set to null for no attachment.",
    )  # noqa: E501

    bcc: Optional[list[Any]] = Field(
        default=[], description="Blind Carbon Copy (BCC) recipients' email addresses."
    )  # noqa: E501

    body: str = Field(
        description="Email content (plain text or HTML); if HTML, `is_html` must be `True`."
    )  # noqa: E501

    cc: Optional[list[Any]] = Field(
        default=[], description="Carbon Copy (CC) recipients' email addresses."
    )  # noqa: E501

    extra_recipients: Optional[list[Any]] = Field(
        default=[],
        description="Additional 'To' recipients' email addresses (not Cc or Bcc).",
    )  # noqa: E501

    is_html: Optional[bool] = Field(
        default=False, description="Set to `True` if the email body contains HTML tags."
    )  # noqa: E501

    recipient_email: str = Field(description="Primary recipient's email address.")  # noqa: E501

    subject: Optional[str] = Field(default=None, description="Subject line of the email.")  # noqa: E501

    user_id: Optional[str] = Field(
        default="me",
        description="User's email address; the literal 'me' refers to the authenticated user.",
    )  # noqa: E501


class CreateEmailDraftInput(BaseModel):
    """Input model for GMAIL_CREATE_EMAIL_DRAFT"""

    attachment: Optional[dict[str, Any]] = Field(
        default=None, description="File to attach to the email."
    )  # noqa: E501

    bcc: Optional[list[Any]] = Field(
        default=[], description="'Bcc' (blind carbon copy) recipient email addresses."
    )  # noqa: E501

    body: str = Field(
        description="Email body content (plain text or HTML); `is_html` must be True if HTML."
    )  # noqa: E501

    cc: Optional[list[Any]] = Field(
        default=[], description="'Cc' (carbon copy) recipient email addresses."
    )  # noqa: E501

    extra_recipients: Optional[list[Any]] = Field(
        default=[], description="Additional 'To' recipient email addresses."
    )  # noqa: E501

    is_html: Optional[bool] = Field(
        default=False,
        description="Set to True if `body` is HTML, otherwise the action may fail.",
    )  # noqa: E501

    recipient_email: str = Field(description="Primary recipient's email address.")  # noqa: E501

    subject: str = Field(description="Email subject line.")  # noqa: E501

    thread_id: Optional[str] = Field(
        default=None,
        description="ID of an existing Gmail thread to reply to; omit for new thread.",
    )  # noqa: E501

    user_id: Optional[str] = Field(
        default="me",
        description="User's email address or 'me' for the authenticated user.",
    )  # noqa: E501


class FetchMessageByMessageIdInput(BaseModel):
    """Input model for GMAIL_FETCH_MESSAGE_BY_MESSAGE_ID"""

    format: Optional[str] = Field(
        default="full",
        description="Format for message content: 'minimal' (ID/labels), 'full' (complete data), 'raw' (base64url string), 'metadata' (ID/labels/headers).",
    )  # noqa: E501

    message_id: str = Field(
        description="Unique ID of the email message to retrieve, obtainable from actions like 'List Messages'."
    )  # noqa: E501

    user_id: Optional[str] = Field(
        default="me",
        description="User's email address or 'me' for the authenticated user.",
    )  # noqa: E501


class MoveToTrashInput(BaseModel):
    """Input model for GMAIL_MOVE_TO_TRASH"""

    message_id: str = Field(
        description="Identifier of the email message to move to trash."
    )  # noqa: E501

    user_id: Optional[str] = Field(
        default="me",
        description="User's email address or 'me' for the authenticated user.",
    )  # noqa: E501


class CreateLabelInput(BaseModel):
    """Input model for GMAIL_CREATE_LABEL"""

    label_list_visibility: Optional[str] = Field(
        default="labelShow",
        description="Controls how the label is displayed in the label list in the Gmail sidebar.",
    )  # noqa: E501

    label_name: str = Field(description="The name for the new label.")  # noqa: E501

    message_list_visibility: Optional[str] = Field(
        default="show",
        description="Controls how messages with this label are displayed in the message list.",
    )  # noqa: E501

    user_id: Optional[str] = Field(
        default="me",
        description="The email address of the user in whose account the label will be created.",
    )  # noqa: E501


class PatchLabelInput(BaseModel):
    """Input model for GMAIL_PATCH_LABEL"""

    color: Optional[dict[str, Any]] = Field(
        default=None,
        description="The color to assign to the label. Color is only available for labels that have their `type` set to `user`.",
    )  # noqa: E501

    id: str = Field(description="The ID of the label to update.")  # noqa: E501

    labelListVisibility: Optional[str] = Field(
        default=None,
        description="The visibility of the label in the label list in the Gmail web interface.",
    )  # noqa: E501

    messageListVisibility: Optional[str] = Field(
        default=None,
        description="The visibility of messages with this label in the message list in the Gmail web interface.",
    )  # noqa: E501

    name: Optional[str] = Field(
        default=None, description="The display name of the label."
    )  # noqa: E501

    userId: str = Field(
        description="The user's email address. The special value `me` can be used to indicate the authenticated user."
    )  # noqa: E501


class GetAttachmentInput(BaseModel):
    """Input model for GMAIL_GET_ATTACHMENT"""

    attachment_id: str = Field(description="ID of the attachment to retrieve.")  # noqa: E501

    file_name: str = Field(description="Desired filename for the downloaded attachment.")  # noqa: E501

    message_id: str = Field(
        description="Immutable ID of the message containing the attachment."
    )  # noqa: E501

    user_id: Optional[str] = Field(
        default="me", description="User's email address ('me' for authenticated user)."
    )  # noqa: E501


class DeleteMessageInput(BaseModel):
    """Input model for GMAIL_DELETE_MESSAGE"""

    message_id: str = Field(description="Identifier of the email message to delete.")  # noqa: E501

    user_id: Optional[str] = Field(
        default="me",
        description="User's email address. The special value 'me' refers to the authenticated user.",
    )  # noqa: E501


class RemoveLabelInput(BaseModel):
    """Input model for GMAIL_REMOVE_LABEL"""

    label_id: str = Field(
        description="ID of the user-created label to be permanently deleted; must exist and not be a system label."
    )  # noqa: E501

    user_id: Optional[str] = Field(
        default="me",
        description="User's email address or 'me' for the authenticated user.",
    )  # noqa: E501


class GetProfileInput(BaseModel):
    """Input model for GMAIL_GET_PROFILE"""

    user_id: Optional[str] = Field(
        default="me",
        description="The email address of the Gmail user whose profile is to be retrieved, or the special value 'me' to indicate the currently authenticated user.",
    )  # noqa: E501


class DeleteDraftInput(BaseModel):
    """Input model for GMAIL_DELETE_DRAFT"""

    draft_id: str = Field(
        description="Immutable ID of the draft to delete, typically obtained when the draft was created."
    )  # noqa: E501

    user_id: Optional[str] = Field(
        default="me",
        description="User's email address or 'me' for the authenticated user; 'me' is recommended.",
    )  # noqa: E501


class AddLabelToEmailInput(BaseModel):
    """Input model for GMAIL_ADD_LABEL_TO_EMAIL"""

    add_label_ids: Optional[list[Any]] = Field(
        default=[],
        description="Label IDs to add. For custom labels, obtain IDs via 'listLabels'. System labels (e.g., 'INBOX', 'SPAM') can also be used.",
    )  # noqa: E501

    message_id: str = Field(
        description="Immutable ID of the message to modify (e.g., from 'fetchEmails' or 'fetchMessagesByThreadId')."
    )  # noqa: E501

    remove_label_ids: Optional[list[Any]] = Field(
        default=[],
        description="Label IDs to remove. For custom labels, obtain IDs via 'listLabels'. System labels can also be used.",
    )  # noqa: E501

    user_id: Optional[str] = Field(
        default="me",
        description="User's email address or 'me' for the authenticated user.",
    )  # noqa: E501


class GetContactsInput(BaseModel):
    """Input model for GMAIL_GET_CONTACTS"""

    page_token: Optional[str] = Field(
        default=None,
        description="Token to retrieve a specific page of results, obtained from 'nextPageToken' in a previous response.",
    )  # noqa: E501

    person_fields: Optional[str] = Field(
        default="emailAddresses,names,birthdays,genders",
        description="Comma-separated person fields to retrieve for each contact (e.g., 'names,emailAddresses').",
    )  # noqa: E501

    resource_name: Optional[str] = Field(
        default="people/me",
        description="Identifier for the person resource whose connections are listed; use 'people/me' for the authenticated user.",
    )  # noqa: E501


class ReplyToThreadInput(BaseModel):
    """Input model for GMAIL_REPLY_TO_THREAD"""

    bcc: Optional[list[Any]] = Field(
        default=[],
        description="BCC recipients' email addresses (hidden from other recipients).",
    )  # noqa: E501

    cc: Optional[list[Any]] = Field(
        default=[], description="CC recipients' email addresses."
    )  # noqa: E501

    extra_recipients: Optional[list[Any]] = Field(
        default=[], description="Additional 'To' recipients' email addresses."
    )  # noqa: E501

    is_html: Optional[bool] = Field(
        default=False,
        description="Indicates if `message_body` is HTML; if True, body must be valid HTML, if False, body should not contain HTML tags.",
    )  # noqa: E501

    message_body: str = Field(
        description="Content of the reply message, either plain text or HTML."
    )  # noqa: E501

    recipient_email: str = Field(description="Primary recipient's email address.")  # noqa: E501

    thread_id: str = Field(description="Identifier of the Gmail thread for the reply.")  # noqa: E501

    user_id: Optional[str] = Field(
        default="me",
        description="Identifier for the user sending the reply; 'me' refers to the authenticated user.",
    )  # noqa: E501


class ListLabelsInput(BaseModel):
    """Input model for GMAIL_LIST_LABELS"""

    user_id: Optional[str] = Field(
        default="me",
        description="Identifies the Gmail account (owner's email or 'me' for authenticated user) for which labels will be listed.",
    )  # noqa: E501


class SendDraftInput(BaseModel):
    """Input model for GMAIL_SEND_DRAFT"""

    draft_id: str = Field(description="The ID of the draft to send.")  # noqa: E501

    user_id: Optional[str] = Field(
        default="me",
        description="The user's email address. The special value `me` can be used to indicate the authenticated user.",
    )  # noqa: E501


class ListDraftsInput(BaseModel):
    """Input model for GMAIL_LIST_DRAFTS"""

    max_results: Optional[int] = Field(
        default=1, description="Maximum number of drafts to return per page."
    )  # noqa: E501

    page_token: Optional[str] = Field(
        default="",
        description="Token from a previous response to retrieve a specific page of drafts.",
    )  # noqa: E501

    user_id: Optional[str] = Field(
        default="me",
        description="User's mailbox ID; use 'me' for the authenticated user.",
    )  # noqa: E501


class ListThreadsInput(BaseModel):
    """Input model for GMAIL_LIST_THREADS"""

    max_results: Optional[int] = Field(
        default=10, description="Maximum number of threads to return."
    )  # noqa: E501

    page_token: Optional[str] = Field(
        default="",
        description="Token from a previous response to retrieve a specific page of results; omit for the first page.",
    )  # noqa: E501

    query: Optional[str] = Field(
        default="",
        description="Filter for threads, using Gmail search query syntax (e.g., 'from:user@example.com is:unread').",
    )  # noqa: E501

    user_id: Optional[str] = Field(
        default="me",
        description="The user's email address or 'me' to specify the authenticated Gmail account.",
    )  # noqa: E501


class ModifyThreadLabelsInput(BaseModel):
    """Input model for GMAIL_MODIFY_THREAD_LABELS"""

    add_label_ids: Optional[list[Any]] = Field(
        default=[],
        description="List of label IDs to add to the thread; these labels must exist.",
    )  # noqa: E501

    remove_label_ids: Optional[list[Any]] = Field(
        default=[],
        description="List of label IDs to remove from the thread; these labels must exist.",
    )  # noqa: E501

    thread_id: str = Field(description="Immutable ID of the thread to modify.")  # noqa: E501

    user_id: Optional[str] = Field(
        default="me",
        description="User's email address or 'me' for the authenticated user.",
    )  # noqa: E501
